/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <gridmgr.h>
#include <app_define.h>
#include <app_debug.h>

#include "define.h"
#include "data/app.h"

static char *_text_get(void *data, Evas_Object *obj, const char *part)
{
	struct app_data *adata;

	if (!data) {
		_ERR("Invalid argument.");
		return NULL;
	}
	adata = data;

	if (!strcmp(get_app_id(adata), APP_ID_BROWSER) &&
			!strcmp(part, PART_THUMB_TEXT_BROWSER))
		return strdup(STR_BROWSER);
	else if (strcmp(get_app_id(adata), APP_ID_BROWSER) &&
			!strcmp(part, PART_THUMB_TEXT))
		return strdup(get_app_name(adata));

	return NULL;
}

static Evas_Object *_content_get(void *data, Evas_Object *obj, const char *part)
{
	struct app_data *adata;
	Evas_Object *img, *icon_bg, *text_bg;
	char *icon, *img_path;
	int w, h, r, g, b, a;

	if (!data || !obj) {
		_ERR("Invalid argument.");
		return NULL;
	}
	adata = data;

	if (!strcmp(part, PART_THUMB_ICON)) {
		icon = get_app_icon(adata);

		img = elm_image_add(obj);
		if (!img) {
			_ERR("elm_image_add failed.");
			return NULL;
		}

		if (!icon || !strcmp(icon, STR_NULL))
			img_path = DEFAULT_APP_ICON_PNG;
		else if (!strcmp(get_app_id(adata), APP_ID_BROWSER))
			img_path = BROWSER_ICON_PNG;
		else
			img_path = icon;

		if (!elm_image_file_set(img, img_path, NULL))
			elm_image_file_set(img, DEFAULT_APP_ICON_PNG, NULL);

		elm_image_object_size_get(img, &w, &h);

		if (w <= SIZE_ICON_MAX && h <= SIZE_ICON_MAX)
			elm_image_resizable_set(img, EINA_FALSE, EINA_FALSE);

		return img;
	} else if (!strcmp(part, PART_THUMB_BG)) {
		icon_bg = evas_object_rectangle_add(obj);
		if (!icon_bg) {
			_ERR("elm_rectangle_add failed.");
			return NULL;
		}

		if (!get_app_icon_bg_color(adata, &r, &g, &b, &a)) {
			_ERR("Get color failed.");
			return NULL;
		}

		evas_object_color_set(icon_bg, r, g, b, a);

		return icon_bg;
	} else if (!strcmp(part, PART_TEXT_BG)) {
		text_bg = evas_object_rectangle_add(obj);
		if (!text_bg) {
			_ERR("elm_rectangle_add failed.");
			return NULL;
		}

		if (!get_app_text_bg_color(adata, &r, &g, &b, &a)) {
			_ERR("Get color failed.");
			return NULL;
		}

		evas_object_color_set(text_bg, r, g, b, a);

		return text_bg;
	}

	return NULL;
}

static Eina_Bool _state_get(void *data, Evas_Object *obj, const char *part)
{
	struct app_data *adata;
	char *id;

	if (!data) {
		_ERR("Data is NULL.");
		return EINA_FALSE;
	}

	adata = data;

	id = get_app_id(adata);
	if (!id)
		return EINA_FALSE;

	if (!strcmp(part, STATE_BROWSER))
		return !strcmp(id, APP_ID_BROWSER);
	else if (!strcmp(part, STATE_NOT_BROWSER))
		return strcmp(id, APP_ID_BROWSER);

	return EINA_FALSE;
}

static struct grid_class _myapp_gclass = {
	.item_style = STYLE_MYAPP,
	.text_get = _text_get,
	.content_get = _content_get,
	.state_get = _state_get,
	.del = NULL,
};

static struct grid_class _fav_gclass = {
	.item_style = STYLE_FAVORITE,
	.text_get = _text_get,
	.content_get = _content_get,
	.state_get = _state_get,
	.del = NULL,
};

struct grid_class *get_myapps_grid_class(void)
{
	return &_myapp_gclass;
}

struct grid_class *get_favorite_grid_class(void)
{
	return &_fav_gclass;
}
